/*
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2001-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.altercast;

import javax.servlet.*;
import javax.servlet.http.*;
// import javax.servlet.jsp.*;
import java.io.*;
import java.net.*;
import java.util.*;
import com.adobe.gl.runtime6.utils.*;

/**
 * Support for Adobe AlterCast.
 * AlterCast support occurs in three phases:
 * <p>
 * At design time, the user places objects on the page and binds
 * AlterCast variables embedded into an image to static values or
 * to items selected from content sources.  This causes GoLive to
 * generate calls to the appropriate computeACSubstURL method.
 * <p>
 * At page generation time, the computeACSubstURL calls are evaluated;
 * each is turned into an IMG SRC= attribute value that references the
 * acsubst servlet with URL parameters that encode the image location
 * and variable bindings.
 * <p>
 * At page display time, the browser sends off the IMG SRC requests,
 * which runs the acsubst servlet, which runs the AlterCast program
 * to compute the image.
 */
public abstract class ACSubst {
	/**
	 * Generates an acsubst reference for image LIVESRC and variable
	 * bindings in A (an array of { name, value, name, value, ... }).
	 * <p>
	 * This is the general form; other computeACSubstURL variants
	 * simply call this one.
	 */
	public static String
	computeACSubstURL(GLContext gl, String livesrc, Object[] a) {
		HttpServletRequest req = gl.getRequest();
		/*
		 * Work around a bug in GoLive by using gl.mapPath() to
		 * prefix scheme://host:port to our path.
		 */
		String ctxt = gl.mapPath(req.getContextPath());
		StringBuffer ts = new StringBuffer(ctxt)
			.append("/acsubst?_t=")
			.append(URLEncoder.encode(fixURL(gl, livesrc)));
		for (int i = 0; i < a.length; i += 2) {
			String name = GLNull.nonnull(a[i]).toString(),
				value = GLNull.nonnull(a[1+i]).toString();
			if ("".equals(name))
				continue;
			ts.append("&amp;")
				.append(URLEncoder.encode(name))
				.append('=')
				.append(URLEncoder.encode(value));
		}
		return ts.toString();
	}

	public static String
	computeACSubstURL(GLContext gl, String livesrc, Object n1, Object v1)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1
				}
			);
	}
	public static String
	computeACSubstURL(GLContext gl, String livesrc, Object n1, Object v1,
		Object n2, Object v2)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1, n2, v2
				}
			);
	}
	public static String
	computeACSubstURL(GLContext gl, String livesrc, Object n1, Object v1,
		Object n2, Object v2, Object n3, Object v3)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1, n2, v2, n3, v3
				}
			);
	}
	public static String
	computeACSubstURL(GLContext gl, String livesrc, Object n1, Object v1,
		Object n2, Object v2, Object n3, Object v3,
		Object n4, Object v4)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1, n2, v2, n3, v3, n4, v4
				}
			);
	}
	public static String
	computeACSubstURL(GLContext gl, String livesrc, Object n1, Object v1,
		Object n2, Object v2, Object n3, Object v3,
		Object n4, Object v4, Object n5, Object v5)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1, n2, v2, n3, v3, n4, v4,
				n5, v5
				}
			);
	}
	public static String
	computeACSubstURL(GLContext gl, String livesrc, Object n1, Object v1,
		Object n2, Object v2, Object n3, Object v3,
		Object n4, Object v4, Object n5, Object v5,
		Object n6, Object v6)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1, n2, v2, n3, v3, n4, v4,
				n5, v5, n6, v6
				}
			);
	}
	public static String
	computeACSubstURL(GLContext gl, String livesrc,
		Object n1, Object v1, Object n2, Object v2,
		Object n3, Object v3, Object n4, Object v4,
		Object n5, Object v5, Object n6, Object v6,
		Object n7, Object v7)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1, n2, v2, n3, v3, n4, v4,
				n5, v5, n6, v6, n7, v7,
				}
			);
	}
	public static String
	computeACSubstURL(GLContext gl, String livesrc,
		Object n1, Object v1, Object n2, Object v2,
		Object n3, Object v3, Object n4, Object v4,
		Object n5, Object v5, Object n6, Object v6,
		Object n7, Object v7, Object n8, Object v8)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1, n2, v2, n3, v3, n4, v4,
				n5, v5, n6, v6, n7, v7, n8, v8,
				}
			);
	}
	public static String
	computeACSubstURL(GLContext gl, String livesrc,
		Object n1, Object v1, Object n2, Object v2,
		Object n3, Object v3, Object n4, Object v4,
		Object n5, Object v5, Object n6, Object v6,
		Object n7, Object v7, Object n8, Object v8,
		Object n9, Object v9)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1, n2, v2, n3, v3, n4, v4,
				n5, v5, n6, v6, n7, v7, n8, v8,
				n9, v9,
				}
			);
	}
	public static String
	computeACSubstURL(GLContext gl, String livesrc,
		Object n1, Object v1, Object n2, Object v2,
		Object n3, Object v3, Object n4, Object v4,
		Object n5, Object v5, Object n6, Object v6,
		Object n7, Object v7, Object n8, Object v8,
		Object n9, Object v9, Object n10, Object v10)
	{
		return computeACSubstURL(gl, livesrc,
			new Object[] {
				n1, v1, n2, v2, n3, v3, n4, v4,
				n5, v5, n6, v6, n7, v7, n8, v8,
				n9, v9, n10, v10,
				}
			);
	}

	/**
	 * Computes a file:/// reference for LIVESRC, which it assumes
	 * to be a page-relative or rooted URL naming an actual file.
	 */
	private static String
	fixURL(GLContext gl, String livesrc) {
		String cpath = gl.contextPathFrom(livesrc);
		String p = gl.getApplication().getRealPath(cpath);
		return "file:///" + p;
	}
};
