/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.custommerchant;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import com.adobe.gl.runtime6.utils.*;

public class CMCartAction extends GLAction {
	protected CMCartAction(GLContext gl, CSWCMCart csc, String provider)
		throws GLException
	{
		super(gl, csc);
		if (csc != null) {
			// this.gen = csc.GetGenerator(true);
			this.providerURL = csc.providerURL;
			this.merchantID = csc.merchantID;
			this.shopperID = csc.shopperID;
		}
		else {
			Properties pinfo =
				CMUtils.getProviderInfo(gl, provider);
			this.merchantID = pinfo.getProperty("merchant-id");
			this.providerURL = pinfo.getProperty("url");
			this.shopperID = CMUtils.getShopperID(gl, pinfo);
		}
	}

	private static void
	getRidOfSetupArgs(GLContext gl) {
		HttpServletRequest req = gl.getRequest();
		req.removeAttribute(SETUP_ARGS_NAME);
	}

	private static Hashtable
	getSetupArgs(GLContext gl) {
		HttpServletRequest req = gl.getRequest();
		Hashtable a = (Hashtable) req.getAttribute(SETUP_ARGS_NAME);
		if (a == null)
			req.setAttribute(SETUP_ARGS_NAME, a=new Hashtable());
		return a;
	}

	private static void
	setup(GLContext gl, String successURL, String failURL,
		String provider, String name)
		throws GLException
	{
		Hashtable a = getSetupArgs(gl);
		a.put("$$provider", provider);
		CMUtils.setPlaceholderReturnURLs(a, name, successURL, failURL);
	}

	/**
	 * Sets up the add-to-cart action.
	 * <p>
	 * SKU, DESCRIPTION, UNITPRICE are of type Object because they
	 * are likely to be extracted from database fields.
	 */
	public static void
	setupAddToCart(GLContext gl, String successURL, String failURL,
		String provider, Object sku,
		Object description, Object unitPrice)
		throws GLException
	{
		setup(gl, successURL, failURL, provider, "add-to-cart");
		Hashtable a = getSetupArgs(gl);
		a.put("$$sku", GLNull.nonnull(sku).toString());
		a.put("$$description", GLNull.nonnull(description).toString());
		a.put("$$unitPrice", GLNull.nonnull(unitPrice).toString());
	}

	public static void
	setupClearCart(GLContext gl, String successURL, String failURL,
		Object cswo)
		throws GLException
	{
		if (!(cswo instanceof CSWCMCart))
			GLUtils.cswQuack("CMCartAction.setupClearCart", cswo);
		CSWCMCart csc = (CSWCMCart) cswo;
		setup(gl, successURL, failURL, csc.provider, "clear-cart");
	}

	public static void
	setupForm(GLContext gl, Object cswo)
		throws GLException
	{
		if (cswo != null && !(cswo instanceof CSWCMCart))
			GLUtils.cswQuack("CMCartAction.setupForm", cswo);

		CSWCMCart csc = (CSWCMCart) cswo;
		Hashtable a = getSetupArgs(gl);
		CMCartAction action = new CMCartAction(gl, csc,
			(String) a.get("$$provider"));
		action.sku = (String) a.get("$$sku");
		action.description = (String) a.get("$$description");
		action.unitPrice = (String) a.get("$$unitPrice");
		a.remove("$$sku");
		a.remove("$$description");
		a.remove("$$unitPrice");
		a.remove("$$provider");

		CMUtils.fixupPlaceholderReturnURLs(a, action, gl);

		a.put(INTERNED_ACTION_PARAM_NAME, gl.intern(action));
		if (csc!=null)
			a.put("$cswName", csc.GetName());
		gl.writeSetupHtml(a);
		a = null;
		getRidOfSetupArgs(gl);
	}

	public static void
	setupUpdateTotals(GLContext gl, String successURL, String failURL,
		Object cswo)
		throws GLException
	{
		if (!(cswo instanceof CSWCMCart))
			GLUtils.cswQuack("CMCartAction.setupUpdateTotals", cswo);
		CSWCMCart csc = (CSWCMCart) cswo;
		setup(gl, successURL, failURL, csc.provider,
			"change-cart-props");

		/*
		 * Hardwire the key name (implicitly, "item/sku") and value.
		 */
		Hashtable a = getSetupArgs(gl);
		ContentSourceWrapper items = csc.NestedCSW("item");
		for (items.MoveFirst(); !items.EOF(); items.Move(1)) {
			a.put("$key["+items.AbsolutePosition()+"]",
				items.Value("sku"));
		}
	}

	private Hashtable gen;
	public final String merchantID, providerURL, shopperID;
	public String description, sku, unitPrice;
	private static final String SETUP_ARGS_NAME =
		GLUtils.GL_PREFIX + "CMCartAction.setupArgs";
	private static final String ACTION_PARAM_NAME = "$$action";
	private static final String INTERNED_ACTION_PARAM_NAME = "$csg";
};
