/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.custommerchant;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;
import com.adobe.gl.runtime6.utils.*;

public class CMOrderAction extends GLAction {
	protected CMOrderAction(GLContext gl, CSWCMOrderForm cso,
		String provider)
		throws GLException
	{
		super(gl, cso);
		if (cso != null) {
			// this.gen = cso.GetGenerator(true);
			this.providerURL = cso.providerURL;
			this.merchantID = cso.merchantID;
			this.shopperID = cso.shopperID;
		}
		else {
			Properties pinfo =
				CMUtils.getProviderInfo(gl, provider);
			this.merchantID = pinfo.getProperty("merchant-id");
			this.providerURL = pinfo.getProperty("url");
			this.shopperID = CMUtils.getShopperID(gl, pinfo);
		}
	}

	private static void
	getRidOfSetupArgs(GLContext gl) {
		HttpServletRequest req = gl.getRequest();
		req.removeAttribute(SETUP_ARGS_NAME);
	}

	private static Hashtable
	getSetupArgs(GLContext gl) {
		HttpServletRequest req = gl.getRequest();
		Hashtable a = (Hashtable) req.getAttribute(SETUP_ARGS_NAME);
		if (a == null)
			req.setAttribute(SETUP_ARGS_NAME, a=new Hashtable());
		return a;
	}

	private static void
	setup(GLContext gl, String successURL, String failURL,
		String provider, String name)
		throws GLException
	{
		Hashtable a = getSetupArgs(gl);
		a.put("$$provider", provider);
		CMUtils.setPlaceholderReturnURLs(a, name, successURL, failURL);
	}

	public static void
	setupForm(GLContext gl, Object cswo)
		throws GLException
	{
		if (cswo != null && !(cswo instanceof CSWCMOrderForm))
			GLUtils.cswQuack("CMOrderAction.setupForm", cswo);

		CSWCMOrderForm cso = (CSWCMOrderForm) cswo;
		Hashtable a = getSetupArgs(gl);
		CMOrderAction action = new CMOrderAction(gl,
			(CSWCMOrderForm) cso, (String) a.get("$$provider"));
		a.remove("$$provider");

		CMUtils.fixupPlaceholderReturnURLs(a, action, gl);

		a.put(INTERNED_ACTION_PARAM_NAME, gl.intern(action));
		if (cso!=null)
			a.put("$cswName", cso.GetName());
		gl.writeSetupHtml(a);
		a = null;
		getRidOfSetupArgs(gl);
	}

	public static void
	setupPurchaseNow(GLContext gl, String successURL, String failURL,
		Object cswo)
		throws GLException
	{
		if (!(cswo instanceof CSWCMOrderForm))
			GLUtils.cswQuack("CMOrderAction.setupForm", cswo);
		CSWCMOrderForm cso = (CSWCMOrderForm) cswo;
		setup(gl, successURL, failURL, cso.provider, "purchase");
	}

	public static void
	setupUpdateOrder(GLContext gl, String successURL, String failURL,
		Object cswo)
		throws GLException
	{
		if (!(cswo instanceof CSWCMOrderForm))
			GLUtils.cswQuack("CMOrderAction.setupForm", cswo);
		CSWCMOrderForm cso = (CSWCMOrderForm) cswo;
		setup(gl, successURL, failURL, cso.provider,
			"change-order-form-props");
	}

	private Hashtable gen;
	public final String merchantID, providerURL, shopperID;
	private static final String SETUP_ARGS_NAME =
		GLUtils.GL_PREFIX + "CMOrderAction.setupArgs";
	private static final String INTERNED_ACTION_PARAM_NAME = "$csg";
};
