/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.jdbc;

/**
 * Base class for wrapping java.sql.Connection.
 * Holds a connection object in a private variable;
 * most methods just delegate to the held connection.
 * <p>
 * Methods needing implementations:
 * <pre>
 *	public void close() throws java.sql.SQLException;
 *	public boolean isClosed() throws java.sql.SQLException;
 * </pre>
 * and of course some sort of constructor.
 */
abstract class GLConnectionBase
	implements java.sql.Connection
{
	/*
	 * Implement javax.sql.Connection.
	 */

	public void
	clearWarnings()
		throws java.sql.SQLException
	{
		con.clearWarnings();
	}
	public void
	commit()
		throws java.sql.SQLException
	{
		con.commit();
	}
	public java.sql.Statement
	createStatement()
		throws java.sql.SQLException
	{
		return con.createStatement();
	}
	public java.sql.Statement
	createStatement(int x, int y)
		throws java.sql.SQLException
	{
		return con.createStatement(x, y);
	}
	public boolean
	getAutoCommit()
		throws java.sql.SQLException
	{
		return con.getAutoCommit();
	}
	public java.lang.String
	getCatalog()
		throws java.sql.SQLException
	{
		return con.getCatalog();
	}
	public java.sql.DatabaseMetaData
	getMetaData()
		throws java.sql.SQLException
	{
		return con.getMetaData();
	}
	public int
	getTransactionIsolation()
		throws java.sql.SQLException
	{
		return con.getTransactionIsolation();
	}
	public java.util.Map
	getTypeMap()
		throws java.sql.SQLException
	{
		return con.getTypeMap();
	}
	public java.sql.SQLWarning
	getWarnings()
		throws java.sql.SQLException
	{
		return con.getWarnings();
	}
	public boolean
	isReadOnly()
		throws java.sql.SQLException
	{
		return con.isReadOnly();
	}
	public java.lang.String
	nativeSQL(java.lang.String s)
		throws java.sql.SQLException
	{
		return con.nativeSQL(s);
	}
	public java.sql.CallableStatement
	prepareCall(java.lang.String s)
		throws java.sql.SQLException
	{
		return con.prepareCall(s);
	}
	public java.sql.CallableStatement
	prepareCall(java.lang.String s, int x, int y)
		throws java.sql.SQLException
	{
		return con.prepareCall(s, x, y);
	}
	public java.sql.PreparedStatement
	prepareStatement(java.lang.String s)
		throws java.sql.SQLException
	{
		return con.prepareStatement(s);
	}
	public java.sql.PreparedStatement
	prepareStatement(java.lang.String s, int x, int y)
		throws java.sql.SQLException
	{
		return con.prepareStatement(s, x, y);
	}
	public void
	rollback()
		throws java.sql.SQLException
	{
		con.rollback();
	}
	public void
	setAutoCommit(boolean b)
		throws java.sql.SQLException
	{
		con.setAutoCommit(b);
	}
	public void
	setCatalog(java.lang.String s)
		throws java.sql.SQLException
	{
		con.setCatalog(s);
	}
	public void
	setReadOnly(boolean b)
		throws java.sql.SQLException
	{
		con.setReadOnly(b);
	}
	public void
	setTransactionIsolation(int z)
		throws java.sql.SQLException
	{
		con.setTransactionIsolation(z);
	}
	public void
	setTypeMap(java.util.Map m)
		throws java.sql.SQLException
	{
		con.setTypeMap(m);
	}

	/*
	 * Other methods
	 */

	/**
	 * Crude logger: if logLevel is at least THRESH, log MSG.
	 */
	protected void log(int thresh, String msg) {
		if (logLevel >= thresh)
			System.out.println(msg);
	}

	protected java.sql.Connection con;
	protected static final int logLevel = 9;
}
