/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.lang.*;
import java.net.*;
import java.text.*;
import java.util.*;

/**
 * Wrappers for java.text.DateFormat functions.
 * These offer access to the FULL, LONG, MEDIUM, SHORT date and
 * time formatting variants, but not to the fine-grained control
 * offered by SimpleDateFormat.
 */
public class GLDateFormat {
	/**
	 * Constructor is private because I don't think this ever
	 * need be instantiated.
	 */
	private GLDateFormat() {
	}

	/*
	 * I wonder if we should cache the get*Instance() values...
	 */

	/**
	 * Formats the year/month/day part of date O according to the locale
	 * from GL and the formatting code FC (one of SHORT, LONG, &c.)
	 */
	public static String
	Date(GLContext gl, Object o, int fc) {
		if (!(o instanceof java.util.Date))
			return GLNull.nonnull(o).toString();
		Locale loc = gl.getUserLocale();
		return DateFormat.getDateInstance(fc, loc)
			.format((java.util.Date) o);
	}
	/**
	 * Formats date O according to the locale
	 * from GL and the formatting codes FC{D,T} (one of SHORT, LONG, &c.)
	 */
	public static String
	DateTime(GLContext gl, Object o, int fcd, int fct) {
		if (!(o instanceof java.util.Date))
			return GLNull.nonnull(o).toString();
		Locale loc = gl.getUserLocale();
		return DateFormat.getDateTimeInstance(fcd, fct, loc)
			.format((java.util.Date) o);
	}
	/**
	 * Formats the time-of-day part of date O according to the locale
	 * from GL and the formatting code FC (one of SHORT, LONG, &c.)
	 */
	public static String
	Time(GLContext gl, Object o, int fc) {
		if (!(o instanceof java.util.Date))
			return GLNull.nonnull(o).toString();
		Locale loc = gl.getUserLocale();
		return DateFormat.getTimeInstance(fc, loc)
			.format((java.util.Date) o);
	}
	/**
	 * LONG, &amp;c., just provide the values of DateFormat.LONG, &amp;c.
	 * so that we can use them in GL-generated code without worrying
	 * about including the right header files.
	 */
	public static final int
		DEFAULT = DateFormat.DEFAULT,
		FULL = DateFormat.FULL,
		LONG = DateFormat.LONG,
		MEDIUM = DateFormat.MEDIUM,
		SHORT = DateFormat.SHORT;
};
