/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

import javax.servlet.jsp.*;

/**
 * Subroutines for writing HTML files to a JspWriter.
 */
public class GLHOut {
	private GLHOut() {}

	public static void
	crlf(JspWriter out)
		throws IOException
	{
		out.print("\r\n");
	}

	public static String
	dummy(Object s) {
		return dummy(s, "");
	}

	public static String
	dummy(Object s, String defval) {
		return s!=null ? s.toString() : defval;
	}

	/**
	 * An end tag for the element named ELEM.
	 */
	public static void
	etag(JspWriter out, String elem, int depth)
		throws IOException
	{
		indent(out, depth);
		out.print("</"); out.print(tagOnly(elem)); out.print('>');
		crlf(out);
	}

	/**
	 * &lt;input type='hidden' name='NAME' value='VALUE'>
	 */
	public static void
	hiddenInput(JspWriter out, String name, String value)
		throws IOException
	{
		out.print("<input type='hidden' name='");
		out.print(GLUtils.htmlEncode(name));
		out.print("' value='");
		out.print(GLUtils.htmlEncode(value));
		out.print("'>");
		crlf(out);
	}

	public static void
	indent(JspWriter out, int depth)
		throws IOException
	{
		while (--depth >= 0)
			out.print('\t');
	}

	/**
	 * Print &lt;FIELD>VALUE&lt;/FIELD>\n to OUT.
	 */
	public static void
	oneElement(JspWriter out, String field, Object value, int depth)
		throws IOException
	{
		indent(out, depth);
		out.print('<'); out.print(field); out.print('>');
		out.print(value);
		/* For close tag, strip off attributes if any */
		out.print("</"); out.print(tagOnly(field)); out.print('>');
		crlf(out);
	}

	public static void
	oneMessage(JspWriter out, String msg)
		throws IOException
	{
		oneElement(out, "MESSAGE", GLUtils.htmlEncode(msg), 1);
	}

	/**
	 * The start tag for an element named ELEM.
	 */
	public static void
	stag(JspWriter out, String elem, int depth)
		throws IOException
	{
		indent(out, depth);
		out.print('<'); out.print(elem); out.print('>');
		crlf(out);
	}

	/**
	 * Just the tag (no attributes).
	 */
	public static String
	tagOnly(String tag) {
		int i = tag.indexOf(" ");
		if (i >= 0)
			tag = tag.substring(0, i);
		return tag;
	}
}
