/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

/**
 * Every GoLive Dynamic Content page is encapsulated in a
 * <code>try/finally</code> block (alternately, a &lt;gl:page>
 * JSP custom action element);
 * the <code>finally</code> clause runs <code>gl.closeAll()</code> to clean
 * up content source wrappers, database connections, &amp;c.
 * Objects created by Dynamic Content's server code are automatically
 * registered for cleanup via the <code>gl.registerCloser(GLNeedsCloser)</code>
 * method.<br>
 * To register your own object with this scheme, do something like<br><code>
 * final MyObject o = myobject;<br>
 * gl.registerCloser(<br>
 * &nbsp;	new GLNeedsCloser() {<br>
 * &nbsp;&nbsp;		void close() {<br>
 * &nbsp;&nbsp;&nbsp;		o.whateverCleanupAction();<br>
 * &nbsp;&nbsp;		}<br>
 * &nbsp;	});<br></code>
 */
public interface GLNeedsCloser {
	void close();
};
