/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.lang.*;
import java.net.*;
import java.util.*;

/**
 * The GLNull object is identifiably null (so that you can distinguish
 * it from any legitimate database return value, for example), but
 * it has more convenient behavior than null does.  In particular,
 * ""+GLNull.val yields ""; ""+null yields "null".
 */
public final class GLNull {
	/**
	 * Constructor is private because GLNull is a singleton whose
	 * one individual is statically constructed.
	 */
	private GLNull() {
	}

	/*
	 * -- Object overrides
	 */
	public boolean
	equals(Object o) {
		return o==null || o==val;
	}
	/**
	 * "" (the empty string)
	 */
	public String
	toString() {
		return "";
	}

	/*
	 * -- Static members
	 */
	/**
	 * Turn null into GLNull.val; return any other Object unchanged.
	 */
	public static Object
	nonnull(Object o) {
		return val.equals(o) ? val : o;
	}
	public static final GLNull val = new GLNull();
};
