/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.utils;

import java.io.*;
import java.util.*;

/**
 * Reads application parameters not stored in <code>WEB-INF/web.xml</code>.
 *
 * Some of the parameters for a GoLive Dynamic Content web application,
 * for example, database connect info,
 * want to be managed as separate files rather than
 * as init parameters in WEB-INF/web.xml.  Here we read a java Properties
 * text file that looks like
 *	param1=value1
 *	param2=value2
 *	...
 */
public class GLParamReader {
	/*
	 * -- Constructor (private).
	 * Use the public static method getData() instead.
	 */
	private GLParamReader() {
	}

	/**
	 * Read the file named by FILE and return a Properties object
	 * containing its parameters.
	 */
	public static java.util.Properties
	getData(GLContext gl, File file)
		throws GLException
	{
		return getData(gl, file.getAbsolutePath());
	}

	/**
	 * Read the file named by URI and return a Properties object
	 * containing its parameters.
	 */
	public static java.util.Properties
	getData(GLContext gl, String uri)
		throws GLException
	{
		Properties props = new Properties();
		InputStream is = null;
		gl.runtimeDebugMessage("getData: "+uri);
		try {
			is = new FileInputStream(uri);
			props.load(is);
		}
		catch (Exception e) {
			throw new GLException(
				GLUtils.cat("GLParamReader.getData(",
					uri, "): ", e.toString()));
		}
		finally {
			try {
				if (is != null) is.close();
			} catch (Exception ce) {};
		}
		return props;
	}
};
