/*
 * GoLive
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2000-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

package com.adobe.gl.runtime6.xml;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;
import com.adobe.gl.runtime6.utils.*;

import org.w3c.dom.*;
import javax.xml.transform.*;

public class CSWXML extends CSWBase {

	/*
	 * -- Constructors.
	 */

	public static CSWXML
	GetXMLSource(GLContext gl, String block, String dstype,
		String xml, String name)
		throws GLException, java.io.FileNotFoundException
	{
		/*
		 * Determine from the xds file what the XSL url is.
		 */
		File dsfile = getDataSourceFile(gl, dstype);
		Document dsdoc = GLXML.asDom(gl, dsfile);
		String xsl = dsdoc.getDocumentElement()
			.getAttribute("xslurl");
		if (xsl==null)
			xsl = "";

		/*
		 * Get the XML document.  If we have an XSL document,
		 * use it to transform the XML.  In either case
		 * <code>doc</code> contains the final product.
		 */
		Document doc = GLXML.asDom(gl, xml);
		if (!"".equals(xsl)) {
			Document sdoc = GLXML.asDom(gl, xsl);
			doc = GLXML.transform(xml, doc, sdoc);
		}

		String single = dsdoc.getDocumentElement()
			.getAttribute("single");
		CSWXML csx = new CSWXML(doc, name, "true".equals(single));
		csx._whence = new StringBuffer("CSWXML.GetXMLSource(" /*)*/)
			.append("block ").append(block)
			.append(", dstype ").append(dstype)
			.append(", xml ").append(xml)
			.append(", name ").append(name)
			.append(/*(*/ ")")
			.toString();
		csx.MoveFirst();
		csx.initPosition(gl, block);
		if (gl.runtimeDebug)
			gl.runtimeDebugMessage(csx.toString());
		return csx;
	}

	private CSWXML(Node topnode, String name, boolean single)
		throws GLException
	{
		this(topnode, single ? "/*" : "/*/*", name);
	}

	public CSWXML(Node topnode, String xpath, String name)
		throws GLException
	{
		super();
		setName(name);
		this.topnode = topnode;
		/*
		 * Discover rows.
		 */
		this.data = GLXML.getNodes(topnode, xpath);
		this.nrow = this.lastRec = this.blksize = data.length;
		this.ncol = 1;
		this._whence = new StringBuffer("CSWXML(" /*)*/)
			.append("topnode ").append(topnode)
			.append(", xpath ").append(xpath)
			.append(", name ").append(name)
			.append(/*(*/ ")")
			.toString();
	}

	/*
	 * -- Implement the ContentSourceWrapper interface.
	 */

	public String
	Error(String name) {
		Object o = null;
		try {
			o = Value(name+"/@error");
		} catch (GLException e) {}
		return GLNull.nonnull(o).toString();
	}

	public java.util.Hashtable
	GetGenerator(boolean b) throws GLException {
		throw new GLException("CSWXML: I don't do GetGenerator");
	}
	public String Key() throws GLException {
		throw new GLException("CSWXML: I don't do Key");
	}
	public void
	MoveToKey(java.util.Hashtable a) throws GLException {
		throw new GLException("CSWXML: I don't do MoveToKey");
	}

	public ContentSourceWrapper
	NestedCSW(String name)
		throws GLException
	{
		if (ixOK != ix) {
			/* throw away the cache */
			this.cswCache = new Hashtable();
			this.ixOK = ix;
		}

		CSWXML r = (CSWXML) cswCache.get(name);
		if (r != null)
			return r;
		/* Validate ix */
		if (locValid(ix, 1, false) != RV_YES)
			throw new GLException("index out of range: ix=" +ix);
		r = new CSWXML(data[ix-1], name, GetName()+"\n"+name);
		cswCache.put(name, r);
		return r;
	}

	public String[]
	PrimaryKey()
	{
		return new String[0];
	}

	public Object
	Value(int j)
		throws GLException
	{
		String xpath = this.lastname;
		this.lastname = null;
		if (xpath==null)
			throw new GLException("Value(Z) called illegally");
		Object o = errorValue(ix, xpath);
		if (!GLNull.val.equals(o))
			return o;
		if (locValid(this.ix, j, false) != RV_YES)
			throw new GLException(
				"index out of range: " + this.ix
					+","+j+" length "+nrow);
		if (xpath.indexOf('@') == -1)
			xpath += "/node()";
		try {
			return GLXML.asText(data[this.ix-1], xpath,
				this.serializer());
		}
		catch (TransformerConfigurationException e) {
			throw new GLException(e.toString());
		}
	}

	/*
	 * Other methods.
	 */

	public static File
	getDataSourceFile(GLContext gl, String dstype) {
		String fname = dstype.endsWith(XML_DSUF) ? dstype
			: dstype+XML_DSUF;
		File dir = new File(
			gl.configFileName(GLUtils.GL_PREFIX+"datasources."));
		gl.runtimeDebugMessage("ds directory " + dir.getAbsolutePath()
			+ " file " + fname);
		return new File(dir, fname);
	}

	/**
	 * Retrieves the list of known datasources.
	 */
	public static String[]
	getDataSources(GLContext gl, boolean all) {
		return gl.getDataSources(XML_DSUF, all);
	}

	public String
	toString() {
		return new StringBuffer()
			.append("<p><b>CSW ").append(name).append("</b>\r\n")
			.append(super.toString())
			.append("_whence ").append(_whence).append("<br>\r\n")
			.toString();
	}

	public static final String XML_DSUF = ".xds";
	private String _whence;

	/*
	 * -- Private/protected.
	 */

	protected int
	getColumnByName(String field) {
		/*
		 * Slight abuse of the mechanism: we just remember
		 * the requested field and always return some valid
		 * column number, e.g., 1.
		 */
		this.lastname = field;
		return 1;
	}

	private void
	initPosition(GLContext gl, String records) {
		GLBlockParam pb = new GLBlockParam(records);
		if (!pb.valid || pb.blksize==0) {
			this.blksize = this.nrow;
			return;
		}
		this.blksize = pb.blksize;

		/*
		 * RECORD_KEY[name]
		 */
		final String rkname = GLUtils.rk(GetName());
		String k = gl.getRequest().getParameter(rkname);
		try {
			if (k != null)
				MoveToKey(gl.mkkey(k));
		}
		catch (GLException e) {
			gl.runtimeDebugMessage("initPosition: "+e.toString());
			MoveTo(1);
		}

		/*
		 * RECORD_INDEX[name]
		 */
		int x = -1;
		try {
			final String rxname = GLUtils.rx(GetName());
			x = Integer.parseInt(gl.getRequest()
				.getParameter(rxname));
			MoveTo(x < 1 ? 1 : x < nrow ? x : nrow);
		} catch (Exception e) {}

		/*
		 * {First,Last}RecordOnPage
		 */
		this.firstRec = ix;
		this.lastRec = firstRec + blksize - 1;
		if (lastRec > nrow)
			lastRec = nrow;
	}

	private Transformer
	serializer()
		throws TransformerConfigurationException
	{
		if (_serializer==null)
			_serializer = GLXML.serializer();
		return _serializer;
	}

	/* Cache of nested csws, maintained by NestedCSW(). */
	private Hashtable cswCache;
	/* Each Node corresponds to a row in this. */
	private Node[] data;
	/* cswCache is valid if ixOK==ix. */
	private int ixOK;
	/* Set by getColumnByName(String), used by Value(int). */
	private String lastname;
	/* Context node for which Value() and NestedCSW() queries */
	private Node topnode;

	/*
	 * A building that turns into a bug?  What's fun about that?
	 */
	private Transformer _serializer;
}
