package com.adobe.gl.servlets6.acsubst;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.adobe.altercast.sdk.*;

/**
 * This servlet allows the user to pass in a set of variables as part of the
 * query string of an HTTP request.  It then creates an AlterCast request
 * that causes the variables to be replaced in a template file.
 * The template must be a full path specified as a value to a parameter named
 * 'template'.  Each of the other parameters should be variable name/value
 * pairs.  The variable name/value pairs are passed straight to AlterCast.
 *
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2001-2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */
public class ACSubstServlet extends HttpServlet {
	/**
	 * Init routine called when servlet is loaded (at application
	 * startup).
	 * Determines configuration parameters, such as the name of the
	 * altercast server program and the debug level, from the
	 * &lt;servlet> entry in the web.xml file.
	 *
	 * TODO Someday switch to the Altercast pooling server.
	 */
	public void
	init() {
		ac_exe_name = getInitParameter(
			"com.adobe.gl.ALTERCAST_BINARY_PROGRAM");
		String s = getInitParameter(
			"com.adobe.gl.ALTERCAST_DEBUG_LEVEL");
		if (s!=null) {
			try {
				int n = Integer.parseInt(s.trim());
				ac_debug = n > 0;
			}
			catch (Exception e) {
			}
		}

		if (ac_debug) {
			System.err.print("ACSubstServlet {exe} ");
			System.err.print(ac_exe_name);
			System.err.print(" {debug} ");
			System.err.println(ac_debug);
		}
	}

	/**
	 * Responds to HTTP GET requests.
	 */
	public void
	doGet(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException
	{
		acsubst(req, resp);
	}

	/**
	 * Responds to HTTP POST requests.
	 */
	public void
	doPost(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException
	{
		acsubst(req, resp);
	}

	/**
	 * Called by do{Get,Post} to respond to HTTP GET and POST requests.
	 * Computes an altercast request, sends to the server,
	 * waits for response, sends response on.
	 */
	private void
	acsubst(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException
	{
		StringBuffer accmd = new StringBuffer();
		String template = "";
		boolean debug = ac_debug;

		accmd.append("<commands>\n")
			.append("\t<loadContent out=\"data\">\n")
			.append("\t\t<data>\n");
		for (Enumeration e = request.getParameterNames();
			e.hasMoreElements();)
		{
			String name = (String) e.nextElement();
			String value = request.getParameter(name);
			if ("_t".equals(name))
				template = value;
			else if ("_d".equals(name))
				debug = true;
			else {
				accmd.append("\t\t\t<")
					.append(xmlEncode(name)).append(">");
				accmd.append(xmlEncode(value));
				accmd.append("</")
					.append(xmlEncode(name)).append(">\n");
			}
		}
		accmd.append("\t\t</data>\n");
		accmd.append("\t</loadContent>\n");
		accmd.append("\t<loadContent source=\"")
			.append(xmlEncode(template)).append("\" />\n");
		accmd.append("</commands>\n");

		if (debug) {
			System.err.println("ACSubstServlet request:");
			System.err.println(accmd.toString());
		}

		Server acserv = null;
		Response acresp = null;
		try {
			acserv = AlterCast.createDirectServer(new File(
				ac_exe_name));
			acserv.setLogStream(System.err);
			Request acreq = AlterCast.createRequest();
			acreq.setCommands(accmd.toString());
			acresp = acserv.execute(acreq);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		finally {
			/* If you use the server pool(?), release here */
		}

		try {
			/* Get the contents from AlterCast. */
			ResultContent[] results = acresp.getResults();
			ResultContent result = results[0];
			/* Send them back as the response. */
			response.setContentType(result.getType());
			byte[] body = result.getData();
			if (body==null || body.length <= 0)
				return;
			ServletOutputStream out = response.getOutputStream();
			out.write(body);
			out.close();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Returns S with xml syntax characters replaced by
	 * numeric character references.
	 */
	private static String
	xmlEncode(String s) {
		final int NS = s.length();
	findxchar: {
		for (int i = 0; i < NS; ++i) {
			char c = s.charAt(i);
			if (c=='<' || c=='>' || c=='&' || c=='"' || c=='\'')
				break findxchar;
		}
		return s;
		}

		StringBuffer ts = new StringBuffer();
		for (int i = 0; i < NS; ++i) {
			final char c = s.charAt(i);
			if (c=='<' || c=='>' || c=='&' || c=='"' || c=='\'')
				ts.append("&#").append((int) c).append(';');
			else
				ts.append(c);
		}
		return ts.toString();
	}

	/**
	 * The name of the altercast program to contact.
	 */
	private String ac_exe_name;
	/**
	 * TRUE iff debugging enabled.
	 */
	private boolean ac_debug;
};
