<?php	
$GLparam = array(
	/*
	 * Arrange to send data in the UTF-8 charset.
	 */
	'output_encoding'=>'UTF-8',
	'content_type'=>'text/xml',
);
require_once("../include/utils.runtime6.php");
require_once("../include/custommerchant.runtime6.php");

//*****************************************************************************
//
// info/custommerchant.order.info6.php
//
// GoLive design-time support for CustomMerchant order forms.
//
// ADOBE SYSTEMS INCORPORATED
// Copyright 2002 Adobe Systems Incorporated. All Rights Reserved.
// 
// NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
// License Agreement, Adobe permits you to reproduce and distribute this 
// file only as an integrated part of a web site created with Adobe 
// GoLive software and only for the purpose of enabling your client to 
// display their web site. All other terms of the Adobe license 
// agreement remain in effect.


// -----------------------------------------------------------------------------
// Return an XML document containing information about the CustomMerchant order
// form datasources available.  Valid requests are:
//
//	custommerchant.order.info6.php?datasources=<all|distinct>
//		return a list of e-commerce cart providers
//		ex) .../custommerchant.order.info6.php?datasources=all
//		ex) .../custommerchant.order.info6.php?datasources=distinct
//
//	custommerchant.order.info6.php?provider=<provider-name>
//		return the schema of an order form from a specific provider
//		ex) .../custommerchant.order.info6.php?provider=Adobe

function xheader() {
	setContentTypeHeader("text/xml");
}

// -----------------------------------------------------------------------------
// Write out the list of available providers.  XML format:
//
//		<DATASOURCES>
//			[<DATASOURCE>...]*

function WriteProviders($all)
{
	$handle = opendir(GetDataSourcePath());

	xheader();
	echo "<DATASOURCES>\r\n";

	while ($file = readdir($handle)) {
		if (ereg('(.*)\.ecp$', $file, $regs)) {
			echo "\t<DATASOURCE>",
				($all ? $file : $regs[1]),
				"</DATASOURCE>\r\n";
		}
	}

	echo "</DATASOURCES>\r\n";
}

// -----------------------------------------------------------------------------
// Send query to provider, run results through a transform to make them 
// palatable to GoLive, echo the finished product.  We can fail sending the 
// message, getting the results, or transforming them; in all cases you should 
// see the standard <ERROR Number='x' Description='y'/> message.
//

function TestMe($provider)
{
	$providerInfo = GetProviderInfo($provider);
	$query = "action=get-order-form&shopper-id=test&merchant-id=" . $providerInfo['merchant-id'];
	$xml = GetHTTPResultsAsText($providerInfo['url'], $query);

	xheader();
	if (indexof($xml, "<ERROR")==0) {
		echo $xml;
		return;
	}
	
	$sablotron = xslt_create();
	$result = xslt_process($sablotron, 'arg:/_xml', 'file://' . getcwd() . '/custommerchant.jxform.info6.xsl', NULL, array('/_xml' => $xml));
	if ($result)
		echo $result;
	else
		writeErrorMessage(xslt_errno($sablotron), xslt_error($sablotron));
	xslt_free($sablotron);
}

// -----------------------------------------------------------------------------
// Write out the the schema of an order form.  XML format:
//
//		<STRUCTURE>
//			[<FIELD>
//				<NAME>...
//				[<FRIENDLY-NAME>...]?
//			]*

function WriteOrderSchema($provider)
{
	$providerInfo = GetProviderInfo($provider);
	$query = "action=get-order-schema&merchant-id=" . $providerInfo['merchant-id'];
	$xml = GetHTTPResultsAsText($providerInfo['url'], $query);
	if (ereg("<custommerchant-result>(.*)</custommerchant-result>", $xml, $regs)) {
		echo trim($regs[1]);
	} else {
		echo $xml;	// should be an error message
	}
}

//-----------------------------------------------------------------------------
// MAIN

RejectUnauthorizedCallers();

$provider = GetFormValue("provider");
$datasources = GetFormValue("datasources");
$test = GetFormValue("test");

// writeHeaderComment(GetCurrentURL().'?'.qatostr($HTTP_GET_VARS));
if ($test != "_no_form_value") {
	TestMe($provider);
} else if ($provider != "_no_form_value") {
	WriteOrderSchema($provider);
} else {
	WriteProviders($datasources == "all");
}


?>
