<?php

/*
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

$GLparam = array(
	/*
	 * Prevent utils.runtime6.php from calling session_start(),
	 * because it tends to cause problems when PHP is
	 * misconfigured.
	 */
	'no_session'=>1,
	/*
	 * Arrange to send data in the UTF-8 charset.
	 */
	'output_encoding'=>'UTF-8',
	'content_type'=>'text/html',
);
require_once("../include/utils.runtime6.php");
require_once("troubleshootinginclude.php");
?>
<html>
	<head>
		<meta http-equiv="content-type" content="text/html;charset=utf-8">
		<title><?php echo $title; ?></title>
		<link rel="stylesheet" href="gl.css" type="text/css">
	</head>
	<body>
		<a name="top"></a>
		<h1><?php echo $title; ?></h1>
		<p>This report contains information about the status of your PHP server
		software and the Adobe GoLive files installed on the server.
		Below you will find the results of various tests performed by this
		troubleshooting script.</p>
<p>Please refer to the GoLive User Guide and "readme" file for more information
about installing and configuring your PHP server to work with Adobe GoLive.</p>
<p>Click here for some <a href="troubleshooting2.php">additional troubleshooting tools</a>.  These tools scan your
entire site looking for dynamic content errors in pages.</p>
<p>Click here to run <a href="mysql_testsql.php">test queries directly to MySQL database servers</a>.</p>

<?php
function checkPHPInfo()
{
	$v0 = '4.0.6';
	$v1 = '4.1.0';

	$test = new Test("PHP server information");
	$test->description = "This test checks php server information.
		<p>
		The PHP version must be at least $v0.
		Certain advanced features, such as XML/XSLT processing,
		e-commerce, and Japanese-language support, require
		version $v1.";

	$os = PHP_OS;
	$version = PHP_VERSION;

	$test->details = "<table border='1' cellpadding='4'>\n";
	$test->details .= "<tr><th>Constant</th><th>Value</th></tr>\n";

	if ($version >= $v1)
		$test->Pass("The PHP server version is $version.");
	else if ($version >= $v0) {
		$test->Verify("Check PHP server information.<br>
			If your site does XML/XSLT processing, e-commerce,
			or processes Japanese-language pages, you need
			PHP version $v1 or above.");
	}
	else {
		$test->Fail("PHP version must be at least $v0.");
		$version = "<span class='failed'>$version</span>";
	}
	$test->details .= "<tr><td>PHP_OS</td><td>$os</td></tr>\n";
	$test->details .= "<tr><td>PHP_VERSION</td><td>$version</td></tr>\n";
	$test->details .= "</table>\n";
	$test->details .= "<p>For more information about this server, visit ";
	$test->details .= "<a target='_blank' href='phpinfo.php'>phpinfo.php</a>.\n";

	return $test;
}

function checkPHPIni()
{
	$test = new Test("php.ini settings");
	$test->description = "GoLive's PHP server code assumes certain values for php.ini file settings.\n";
	$test->description .= "This test checks that the settings have the correct values.\n";
	$test->description .= "<p>GoLive-generated pages may operate incorrectly, or fail altogether,\n";
	$test->description .= "if these settings have incorrect values.\n";
	global $SERVER_SOFTWARE;
	global $GL_strlen;
	global $GL_strpos;

	$error = 0;
	$register_globals     = get_cfg_var("register_globals");
	$include_path         = get_cfg_var("include_path");
	$magic_quotes_gpc     = get_cfg_var("magic_quotes_gpc");
	$magic_quotes_runtime = get_cfg_var("magic_quotes_runtime");
	$xtrack_vars          = get_cfg_var("track_vars");
	$session_save_path    = get_cfg_var("session.save_path");

	$test->details = "<table border=\"1\">\n";
	$test->details .= "<tr><th>Constant</th><th>Value</th><th>Note</th></tr>\n";

	$test->details .= "<tr><td>register_globals</td>\n";
	$test->details .= "<td>" . onoff($register_globals, true) . "</td><td>\n";
	if ($register_globals) {
		$test->details .= "<span class=\"passed\">OK</span><br>This setting is correct.";
	} else {
		$test->details .= "<span class=\"failed\">ERROR</span><br>This setting needs to be On.";
		$error++;
	}
	$test->details .= "</td></tr>\n";

	$test->details .= "<tr><td>include_path</td><td>$include_path</td><td>\n";
	if ($GL_strlen($include_path)) {
		$test->details .= "Server software is ($SERVER_SOFTWARE).<br>";
		if ($GL_strpos($SERVER_SOFTWARE, "Apache") !== false) {
			$test->details .= "<span class=\"failed\">ERROR</span><br>";
			$test->details .= "With Apache this setting needs be empty or unspecified.";
			$error++;
		} else if ($GL_strpos($SERVER_SOFTWARE, "IIS") !== false) {
			$test->details .= "<span class=\"verify\">VERIFY</span><br>";
			$test->details .= "With IIS this setting works, but with Apache it won't work.";
			$verify++;
		} else {
			$test->details .= "<span class=\"verify\">VERIFY</span><br>";
			$test->details .= "With IIS this setting works, but with Apache it won't work.";
		}
	} else {
		$test->details .= "<span class=\"passed\">OK</span><br>This setting is correct.";
	}
	$test->details .= "</td></tr>\n";

	$test->details .= "<tr>\n";
	$test->details .= "<td>magic_quotes_gpc</td>\n";
	$test->details .= "<td>" . onoff($magic_quotes_gpc, true) . "</td>\n";
	$test->details .= "<td>\n";
	if ($magic_quotes_gpc) {
		$test->details .= "<span class=\"passed\">OK</span><br>This setting is correct.";
	} else {
		$test->details .= "<span class=\"failed\">ERROR</span><br>This setting needs to be On.";
		$error++;
	}
	$test->details .= "</td></tr>\n";

	$test->details .= "<tr>\n";
	$test->details .= "<td>magic_quotes_runtime</td>\n";
	$test->details .= "<td>" . onoff($magic_quotes_runtime, false) . "</td>\n";
	$test->details .= "<td>\n";
	if ($magic_quotes_runtime) {
		$test->details .= "<span class=\"failed\">ERROR</span><br>This setting needs to be Off.";
		$error++;
	} else {
		$test->details .= "<span class=\"passed\">OK</span><br>This setting is correct.";
	}
	$test->details .= "</td></tr>\n";

	$test->details .= "<tr><td>session.save_path</td><td>$session_save_path</td><td>\n";
	if ((PHP_OS == "WINNT")&&(!$GL_strpos($session_save_path, ":"))) {
		$test->details .= "<span class=\"failed\">ERROR</span><br>The path setting is wrong. Please specify valid path.";
		$error++;
	} else {
		$test->details .= "<span class=\"passed\">OK</span><br>This setting is correct.";
	}
	$test->details .= "</td></tr>\n";

	$test->details .= "<tr>\n";
	$test->details .= "<td>track_vars</td>\n";
	$test->details .= "<td>" . onoff($xtrack_vars) . "</td><td>&nbsp;</td>\n";
	$test->details .= "</tr></table>\n";

	$test->details .= "<p>For more information about this server, visit\n";
	$test->details .= "<a target='_blank' href='phpinfo.php'>phpinfo.php</a>.\n";

	if ($error == 0 ) {
		$test->Pass("php.ini settings seem to be fine.");
	} else {
		$test->Fail("Check php.ini file settings.");
	}
	return $test;
}


function directoryTree($folder, $level)
{
	$msg = "";
	$dir = opendir($folder);
	$children = array();
	while ($file = readdir($dir)) {
		if ($file != '.' && $file != '..')
			$children[] = $file;
	}
	closedir($dir);
	usort($children, 'fnamecmp');

	for ($c = 0; $c < count($children); ++$c) {
		$file = $children[$c];
		$fullpath = $folder . "/" . $file;
		for ($i = 0; $i < $level; ++$i)
			$msg .= "    ";
		if (is_dir($fullpath)) {
			$msg .= "<b>$file {</b>\r\n";
			$msg .= directoryTree($fullpath, 1+$level);
			for ($i = 0; $i < $level; ++$i)
				$msg .= "    ";
			$msg .= "<b>}</b>\r\n";
		}
		else {
			$msg .= $file;
			$msg .= "\r\n";
		}
	}
	return $msg;
}

function checkPHPFunctions() {
	$test = new Test("Check PHP builtin functions");
	$test->description = "This test checks that certain functions are supplied by the PHP installation.\n";
	$test->description .= "GoLive-generated pages may operate incorrectly, or fail altogether,\n";
	$test->description .= "if these functions are not available.\n";


	$checkFuncs = array(
		array('xslt_process', 0,
			'Used by XML content sources and e-commerce.'),
		array('curl_exec', 0,
			'Used to send HTTPS requests to e-commerce providers.'),
		array('openssl_open', 0,
			'Used to read HTTPS responses from an e-commerce provider.'),
		// array('not really a function', 1,
		// 	'Used to demonstrate a failure of this test.'),
	);
	$fx = 0; $rx = 1; $nx = 2;
	$error = false;
	$verify = false;

	$test->details .= "<table border='1' cellpadding='4'><tr>
		<th>Name</th><th>Found</th><th>Note</th></tr>";
	for ($i = 0; $i < count($checkFuncs); ++$i) {
		$name = $checkFuncs[$i][$fx];
		$required = $checkFuncs[$i][$rx];
		$note = $checkFuncs[$i][$nx];
		if (function_exists($name))
			$found = "yes";
		else if (!$required) {
			$verify = true;
			$found = "<span class='verify'>NO</span>";
		}
		else {
			$error = true;
			$found = "<span class='failed'>NO</span>";
		}
		$test->details .= "<tr><td>$name</td><td>$found</td>
			<td>$note</td></tr>\r\n";
	}
	$test->details .= "</table>\r\n";
	if ($error)
		$test->Fail("The PHP installation is missing one or more
			required functions.");
	else if ($verify)
		$test->Verify("The PHP installation is missing one or more
			special-purpose functions (e.g., functions used in
			XML processing, e-commerce).
			Verify that your site does not need these functions.");
	else
		$test->Pass("All required functions are present.");
	return $test;
}


function nonempty($s, $def="&nbsp;") {
	return IsEmpty($s) ? $def : $s;
}

function checkMultiByteSupport() {
	$test = new Test("Check multibyte support");
	$test->description =
"This test checks that the PHP installation enables PHP multibyte support."
. "If multibyte character support is not enabled, GoLive will not "
. "be able to handle sites with non-Western European characters "
. "in database table/field names or in XML content sources.";

	if (function_exists('mb_strlen')) {
		$error = false;
		$output_buffering        = get_cfg_var('output_buffering');
		$default_charset         = get_cfg_var('default_charset');
		$mb_detect_order         = get_cfg_var('mbstring.detect_order');
		$mb_http_input           = get_cfg_var('mbstring.http_input');
		$mb_http_output          = get_cfg_var('mbstring.http_output');
		$mb_internal_encoding    = get_cfg_var('mbstring.internal_encoding');
		$mb_substitute_character = get_cfg_var('mbstring.substitute_character');

		$test->details = "<table border='1' cellpadding='4'>\n";
		$test->details .= "<tr><th>Constant</th><th>Value</th></tr>\n";
		$test->details .= "<tr><td>output_buffering</td><td>"
			. nonempty($output_buffering) . "</td></tr>\r\n";
		$test->details .= "<tr><td>default_charset</td><td>"
			. nonempty($default_charset) . "</td></tr>\r\n";
		$test->details .= "<tr><td>mbstring.detect_order</td><td>"
			. nonempty($mb_detect_order) . "</td></tr>\r\n";
		$test->details .= "<tr><td>mbstring.http_input</td><td>"
			. nonempty($mb_http_input) . "</td></tr>\r\n";
		$test->details .= "<tr><td>mbstring.http_output</td><td>"
			. nonempty($mb_http_output) . "</td></tr>\r\n";
		$test->details .= "<tr><td>mbstring.internal_encoding</td><td>"
			. nonempty($mb_internal_encoding) . "</td></tr>\r\n";
		$test->details .= "<tr><td>mbstring.substitute_character</td><td>"
			. nonempty($mb_substitute_character) . "</td></tr>\r\n";
		$test->details .= "</table>\r\n";
	} else {
		$error = true;
	}

	if ($error)
		$test->Verify("The PHP installation does not support multibyte functions.");
	else
		$test->Pass("The PHP installation supports multibyte functions.");
	return $test;
}

function checkServerConfigFiles()
{
	$test = new Test("Check server config files");

	$test->description = "This test checks to see whether the PHP and XML files for GoLive\n";
	$test->description .= "have been properly installed on the server.\n";
	$test->description .= "It also produces a directory tree list of all files in the config folder.\n";
	$test->description .= "The following files are required.\n";

	$thisFile = GetCurrentFile();
	$test->details = "This file path = [$thisFile]<br>\n";
	$dirname = dirname($thisFile);
	$test->details .= "This directory = [$dirname]<br>\n";
	$configPath = dirname($dirname);
	$test->details .= "Config Path = [$configPath]<p>\n";

	$stride = 3;	//	name, required, note
	$checkFiles = Array(
		"actions/glerror.php",					0, "error page for failed actions",
		"actions/custommerchant.actions6.php",	0, "for CustomMerchant",
		"actions/mysql.actions6.php",			0, "for MySQL updating",
		"include/friends.php",					1, "required",
		"include/custommerchant.runtime6.php",	1, "required",
		"include/mysql.runtime6.php",			1, "required",
		"include/utils.runtime6.php",			1, "required",
		"include/xml.runtime6.php",				1, "required",
		"info/CMDataSourceEditor/Main.html",	0, "for CustomMerchant authoring",// shared
		"info/MySQLDataSourceEditor/Main.html",	0, "for MySQL PHP authoring",
		"info/XMLDataSourceEditor/Main.html",	0, "for XML authoring",			// shared
		"info/general.ui6.xml",					0, "for authoring",				// shared
		"info/custommerchant.cart.info6.php",	0, "for CustomMerchant",
		"info/custommerchant.order.info6.php",	0, "for CustomMerchant",
		"info/custommerchant.ui6.xml",			0, "for CustomMerchant",			// shared
		"info/custommerchant.xform.info6.xsl",	0, "for CustomMerchant",			// shared
		"info/mysql.info6.php",					0, "for MySQL PHP authoring",
		"info/mysql.ui6.xml",					0, "for MySQL PHP authoring",
		"info/parsephp.php",					0, "for PHP authoring",
		"info/reservedwords.ui6.xml",			0, "for PHP authoring",
		"info/test.php",						0, "for PHP authoring",
		"info/test.html",						0, "for PHP authoring",			// shared
		"info/mysql_testsql.php",				0, "for PHP authoring",
		"info/troubleshooting.php",				0, "this page",
		"info/ui6.php",							0, "for PHP authoring",
		"info/php.ui6.xml",						0, "for PHP authoring",
		"info/xml.info6.php",					0, "for XML authoring",
		"info/xml.ui6.xml",						0, "for XML authoring");		// shared

	$error = 0;
	$verify = 0;

	$test->details .= "<table border='1' cellpadding='4'>\n";
	$test->details .= "<tr><th>File</th><th>Found</th><th>Note</th></tr>\n";
	for ($i = 0; $i < count($checkFiles); $i += $stride) {
		$name     = $checkFiles[$i];
		$required = $checkFiles[$i+1];
		$note     = $checkFiles[$i+2];

		$filePath = $configPath . "/" . $name;
		$test->details .= "<tr><td>$name</td><td>";
		if (file_exists($filePath)) {
			$test->details .= "yes";
		} else {
			if ($required) {
				$test->details .=
					"<span class='failed'>NO</span>";
				$error++;
			} else {
				$test->details .=
					"<span class='verify'>NO</span>";
				$verify++;
			}
		}
		$test->details .= "</td><td>$note</td></tr>\n";
	}
	$test->details .= "</table>\n";
	$test->details .= "<p>Config files:<br>\n";
	$test->details .= "<pre>\n";
	$test->details .= "<b>$configPath {</b>\r\n";
	$test->details .= directoryTree($configPath, 1);
	$test->details .= "<b>}</b>\r\n";
	$test->details .= "</pre>\n";

	$nmiss = $error + $verify;
	if ($error) {
		$test->Fail($nmiss . " file(s) are missing from your config folder.");
	} else {
		if ($verify == 0) {
			$test->Pass("All server configuration files are present.");
		} else {
			$test->Verify($nmiss . " file(s) are missing from your config folder.");
		}
	}
	return $test;
}

function checkFileWrite(&$test, $filePath) {
	$test->details = "Test file = [$filePath]<p>\n";
	$error = 0;
	$checker = 0;
	if (file_exists($filePath)) {
		$error = -1;
	}
	else if (!($fp = @fopen($filePath, "w"))) {
		$error++;
		$test->details .= "<span class='failed'>File operation failed to create.</span>\n";
	}
	else {
		if (!fputs($fp, "GoLive Dynamic Content")) {
			$error++;
			$test->details .= "<span class='failed'>File operation failed to write.</span>\n";
		}
		if (!fclose($fp)) {
			$error++;
			$test->details .= "<span class='failed'>File operation failed to close.</span>\n";
		}
		if (!unlink($filePath)) {
			$error++;
			$test->details .= "<span class='failed'>File operation failed to delete.</span>\n";
		}
	}
	if ($error == 0) {
		$test->Pass("File writing test was successful.");
	} else if ($error == -1) {
		$test->Fail("The test filename already exists.");
	} else {
		$test->Fail("File writing test failed.");
	}
}

function checkServerConfigWritePrivilege()
{
	$test = new Test("Check config folder write permission");

	$test->description = "This test checks the write permission of the config folder,\n";
	$test->description .= "which is used by GoLive's Preview function.\n";
	$test->description .= "To preview a dynamic page, GoLive uploads a temporary file to the\n";
	$test->description .= "config folder so that the web server can process it.\n";
	$test->description .= "If the folder does not have write permission, the preview will not work.\n";
	$test->description .= "Note that you may want to disable write permission on your production servers.\n";
	$test->description .= "In that case, you will not be able to use the GoLive preview with the\n";
	$test->description .= "public server as your Config URL.\n";
	$test->description .= "Other features will not be affected.\n";

	$thisFile = GetCurrentFile();
	$dirname = dirname($thisFile);
	$configPath = dirname($dirname);
	$filePath = $configPath . "/writingtest.txt";
	checkFileWrite($test, $filePath);
	return $test;
}


function checkSessionFileWritePrivilege() {
	global $Test_Fail;
	global $GL_strlen;

	$test = new Test("Check session file folder write permission");

	$test->description = "This test checks that the directory into which PHP writes its session\n";
	$test->description .= "data is present and that this application can write temporary files in it.\n";
	$test->description .= "Web applications that depend on session data will not function correctly\n";
	$test->description .= "if this directory is not writable.\n";

	$session_save_path = get_cfg_var("session.save_path");
	if ($GL_strlen($session_save_path) == 0) {
		$test->Fail("session.save_path setting is wrong.");
	} else {
		$filePath = $session_save_path . "/writingtest.txt";
		checkFileWrite($test, $filePath);
	}
	if ($test->result == $Test_Fail) {
		$test->details = "<p>\n";
		$test->details .= "To remedy: change the <b>php.ini</b> <code>session.save_path</code>\n";
		$test->details .= "setting to the name of a directory that exists and is writable.\n";
	}
	return $test;
}


function startElement($parser, $name, $attrs) {
}

function endElement($parser, $name) {
}


function checkServerXMLFileFormat()
{
	$test = new Test("Check XML files");

	$test->description  = "This test performs a basic syntax check
		on the XML files in the config/info folder.\r\n";

	$thisFile = GetCurrentFile();
	$dirname = dirname($thisFile);
	$configPath = dirname($dirname);
	$folder = $configPath . "/info";
	$error = 0;

	/* Discover and sort list of filenames. */
	$handle = opendir($folder);
	$xfiles = array();
	while ($file = readdir($handle)) {
		if (ereg('.*\.ui[0-9]+\.xml$', $file))
			$xfiles[] = $file;
	}
	closedir($handle);
	usort($xfiles, 'fnamecmp');

	$test->details = "<table border='1' cellpadding='4'>\n";
	$test->details .= "<tr><th>XML File</th><th>Package version</th><th>Check</th></tr>\n";

	/* Process files. */
	for ($xi = 0; $xi < count($xfiles); ++$xi) {
		$file = $xfiles[$xi];
		if (!ereg('(.*)\.ui([0-9]+)\.xml$', $file, $regs))
			continue;
		$thisPackageVersion = $regs[2];
		if ($thisPackageVersion > 100) {
			$thisPackageVersion /= 100;
		} else if ($thisPackageVersion > 10) {
			$thisPackageVersion /= 10;
		}

		$fullpath = $folder . "/" . $file;
		$test->details .= "<tr><td>$file</td><td>$thisPackageVersion</td><td>\n";
		$thisFileError = 0;
		if (!($fp = fopen($fullpath, "r"))) {
			++$error;
			++$thisFileError;
			$test->details .= "<span class='failed'>Could not open XML input.</span><br>\n";
			continue;
		}

		$xml_parser = xml_parser_create();
		xml_set_element_handler($xml_parser, "startElement", "endElement");

		while ($data = fread($fp, 4096)) {
			if (!xml_parse($xml_parser, $data, feof($fp))) {
				$test->details .= sprintf( "<span class='failed'>XML error: %s at line %d.</span><br>\n",
					xml_error_string(xml_get_error_code($xml_parser)),
					xml_get_current_line_number($xml_parser));
				$error++;
				$thisFileError++;
			}
		}
		xml_parser_free($xml_parser);
		fclose($fp);

		if ($thisFileError == 0) {
			$test->details .= "<span class='passed'>OK</span><br>\n";
		}
		$test->details .= "</td></tr>\n";
	}
	$test->details .= "</table>\n";

	if ($error == 0) {
		$test->Pass("All XML files passed.");
	} else {
		$test->Fail("One or more XML file(s) have problem(s).");
	}
	return $test;
}


function checkServerConfigFilesReturnCode()
{
	$test = new Test("Check config file's line endings");

	$test->description = "This test checks whether the GoLive text files have proper line endings.\n";
	$test->description .= "(Specifically, it checks the friends.php file.)\n";
	$test->description .= "If you upload config files or other PHP files from a Macintosh using\n";
	$test->description .= "FTP binary or 'image' upload, PHP may not work properly.\n";
	$test->description .= "You must upload PHP files as text.\n";
	$test->description .= "<p>\n";
	$test->description .= "Also, if the files don't have Windows-style (carriage return followed\n";
	$test->description .= "by line feed) line endings, PHP pages may not work on Windows-based servers.\n";
	$test->description .= "You can set the line endings style for uploaded files in GoLive under\n";
	$test->description .= "Edit&gt;Web Settings, Global tab, and select <code>Windows (CR/LF)</code>\n";
	$test->description .= "from the Line Break Character popup menu.\n";
	$test->description .= "<p>\n";
	$test->description .= "Similarly, if the server is UNIX-based (including Mac OS X),\n";
	$test->description .= "the files should have UNIX (LF) line endings.\n";

	$thisFile = GetCurrentFile();
	$dirname = dirname($thisFile);
	$configPath = dirname($dirname);
	$filePath = $configPath . "/include/friends.php";

	$test->details = "Checking file [$filePath]:\n";

	if (file_exists($filePath)) {
		$a = GL_file($filePath);
		$lineCounter = count($a);
		$test->details .= "$lineCounter line(s)<p>\n";
		if ($lineCounter == 0) {
			$test->Fail("The file is empty.");
		} else if ($lineCounter == 1) {
			$test->Fail("The file has no line breaks. If the page was uploaded from a Mac, make sure it is uploaded as text.");
		} else {
			$test->Pass("The file was uploaded correctly.");
		}
	} else {
		$test->details .=
			"$filePath was not found<br>\n";
		$test->Fail("Server configuration files have not been uploaded properly.");
	}
	return $test;
}


function checkServerConfigFriendsFile()
{
	global $GL_strlen;
	$test = new Test("Check config/include/friends.php file");

	$test->description = "This test checks the friends.php file, which is used to provide\n";
	$test->description .= "IP-based security.\n";
	$test->description .= "By default, access to the GoLive Dynamic Content authoring features\nis allowed only from the machine running the server ('localhost').\n";
	$test->description .= "To allow access from another host, you must edit the\n";
	$test->description .= "config/include/friends.php file (or use the Dynamic Content \nsetup wizard) to include the IP address of that machine.\n";

	$thisFile = GetCurrentFile();
	$dirname = dirname($thisFile);
	$configPath = dirname($dirname);
	$filePath = $configPath . "/include/friends.php";

	$test->details = "Checking file = [$filePath]<p>\n";

	$error = 0;
	$checker = 0;

	if (file_exists($filePath)) {
		$fileLines = GL_file($filePath);
		$test->details .= "<table border='1' cellpadding='4'>\n";
		$test->details .= "<tr><th>IP address</th><th>Mask</th><th>Note</th></tr>\n";
		for ($i = 0; $i < count($fileLines); ++$i) {
			$line = $fileLines[$i];
			if ($GL_strlen($line)<4 || $line[0]==";" || $line[0]=="<")
				continue;

			// ToDO: update utils.php to use split.
			$splitline = split("[ \t]+", trim($line));
			$addrStr = explode(".", $splitline[0]);
			$maskStr = (count($splitline) > 1)
				? explode(".", $splitline[1])
				: Array("255", "255", "255", "255");
			$addr[0] = (int)$addrStr[0];
			$addr[1] = (int)$addrStr[1];
			$addr[2] = (int)$addrStr[2];
			$addr[3] = (int)$addrStr[3];
			$mask[0] = (int)$maskStr[0];
			$mask[1] = (int)$maskStr[1];
			$mask[2] = (int)$maskStr[2];
			$mask[3] = (int)$maskStr[3];
			// $as,$ms are now canonical dotted-decimal IP addrs
			$as = join('.', $addr);
			$ms = join('.', $mask);
			$test->details .= "<tr><td>$as</td><td>$ms</td><td>";

			if ($as == '127.0.0.1' && $ms == '255.255.255.255') {
				$test->details .=
					"This is the default setting.\n";
				if ($checker == 0)
					$checker = 1;
			}
			else if ($ms == '0.0.0.0') {
				$test->details .=
					"This setting disables all security.\n";
				$checker = 2;
			}
			else if (~0xff & ($addr[0]|$addr[1]|$addr[2]|$addr[3]
					|$mask[0]|$mask[1]|$mask[2]|$mask[3]))
			{
				$test->details .= "<span class='failed'>This is an invalid setting.</span>\n";
				$test->details .= "Each section of IP address and mask must be between 0 and 255.\n";
				$checker = 4;
			}
			else {
				$test->details .= "Other settings<br>\n";
				if ($checker < 2) {
					$checker = 3;
				}
			}
			$test->details .= "</td></tr>\n";
		}
		$test->details .= "</table>\n";
		switch ($checker) {
		case 0:
			$test->Fail('The friends.php file contains no IP addresses for access control.');
			break;
		case 1:
			$test->Verify('The friends.php file is the same as the installation default. It will allow access only to the local host.');
			break;
		case 2:
			$test->Verify('The friends.php file is set as "disable all security".');
			break;
		case 3:
			$test->Pass("Server configuration is done.");
			break;
		default:
			$test->Fail('Server configuration files are not set up properly.');
			break;
		}
	}
	else {
		$test->details .= "config/include/friends.php was not found<br>\n";
		$test->Fail("Server configuration files were not uploaded properly.");
	}
	return $test;
}

function asString($a) {
	$s = '';
	if (is_array($a)) {
		reset($a);
		while (list($k,$v)=each($a)) {
			$s .= $k.'='.$v."<br>\n";
		}
	}
	return $s;
}

function checkServerMySQL()
{
	$test = new Test("Check MySQL database setting");

	$test->description = "This test checks the MySQL database setting on the server.\n";

	$test->details = "";
	$error = 0;
	$verify = 0;

//	$db_list = mysql_list_dbs();
	$handle = opendir(GetDataSourcePath());

	$test->details .= "<table border='1'>\n";
	$test->details .= "<tr><th>Data Source</th><th>DL Info</th><th>Table/Field</th></tr>";

	while ($file = readdir($handle)) {
		if (ereg('(.*)\.mysql\.sbs$', $file, $regs)) {
			$dataSource = $regs[1];
		} else {
			continue;
		}

		global $GL_mysql_dsnames;
		global $GL_mysql_dsnames_result;
		global $GL_strlen;
		global $GL_strpos;

		$r = @$GL_mysql_dsnames[$dataSource];
		if (!isset($r)) {
			$r = GL_mysql_get_dsinfo($dataSource);
			$GL_mysql_dsnames[$dataSource] = $r;
		}
		$errMsg = "";
		$GL_mysql_dsnames_result = $r;
		if (isset($r) && $r != null && is_array($r)) {
			$linkId = mysql_connect(@$r['host'], @$r['user'], @$r['password']) or
				$errMsg .= 'ERROR: Could not connect to host [' . @$r['host'] . '].<br>';
		} else {
			$linkId = mysql_connect('localhost') or 
				$errMsg .= 'ERROR: Could not connect host [localhost].<br>';
		}
		if (!mysql_select_db(@$r['database'])) {
			$errMsg .= 'ERROR: Could not find database [' . $r['database'] . '].<br>';
		}
		$database = $GL_mysql_dsnames[$dataSource]['database'];
		$test->details .= "<tr>\n";
		$test->details .= "<td>$dataSource</td>\n";
		$test->details .= "<td>" . asString($GL_mysql_dsnames_result);
		if ($GL_strlen($errMsg) != 0) {
			$test->details .= "<span class='failed'>$errMsg</span></td><td></td></tr>\n";
			++$error;
			continue;
		}
		$test->details .= "</td>\n<td>\n";
		$table_list = mysql_list_tables($database, $linkId);
		if ($table_list==null) {
			$test->details .= "<span class='verify'>NO DATA</span></td></tr>\n";
			++$verify;
			continue;
		}
		$test->details .= "<dl>\n";
		for ($i = 0; $i < mysql_num_rows($table_list); $i++ ) {
			$tableName = mysql_tablename($table_list, $i);
			$test->details .= "<dt>$tableName\n";
			$query = "SELECT * FROM " . $tableName;
			$result = mysql_query($query);
			/* $table = mysql_field_table ($result, $i); */
			$test->details .= "<dd><table border='1'>\n";
			$test->details .= "<tr><th>Name</th><th>Type</th><th>Length</th><th>Flags</th></tr>\n";
			$foundPrimaryKey = 0;
			for ($j = 0; $j < mysql_num_fields($result); $j++) {
				$name = mysql_field_name($result, $j);
				$type = mysql_field_type($result, $j);
				$len = mysql_field_len($result, $j);
				$flags = mysql_field_flags($result, $j);
				$test->details .= "<tr><td>$name</td><td>$type</td><td>$len</td><td>$flags</td></tr>\n";
				if ($GL_strpos($flags, "primary_key")) {
					$foundPrimaryKey++;
				}
			}
			$test->details .= "</table>\n";
			if ($foundPrimaryKey == 0){
				$test->details .= "<dd><span class='verify'>This table does not have a primary key field.</span><p>\n";
				$verify++;
			} else {
				$test->details .= "<dd><span class='passed'>Found primary key field(s).</span><p>\n";
			}
			mysql_free_result($result);
		}
		mysql_free_result($table_list);
		mysql_close($linkId);
		$test->details .= "</dl></td></tr>\n";
	}
	$test->details .= "</table>\n";

	if ($error == 0) {
		if ($verify == 0) {
			$test->Pass("MySQL settings seem to be fine.");
		} else {
			$test->Verify("MySQL settings may have problem(s).");
		}
	} else {
		$test->Fail("Problem(s) were found in MySQL settings.");
	}

	return $test;
}


function checkServerEcpFiles()
{
	$test = new Test("Check CustomMerchant datasource files");

	$test->description = "This test checks the CustomMerchant datasource file ";
	$test->description .= "in config/datasources folder.\n";

	$test->details = "";
	$error = 0;
	$counterEcp = 0;
	$errStr = "<span class=\"Failed\">ERROR</span><br>\n";

	$test->details .= "<table border='1'>\n";
	$test->details .= "<tr><th>Data Source</th><th>Setting</th></tr>";

	$handle = opendir(GetDataSourcePath());
	while ($file = readdir($handle)) {
		if (ereg('(.*)\.ecp$', $file, $regs)) {
			$dataSource = $regs[1];
			$counterEcp++;
		} else {
			continue;
		}

		$test->details .= "<tr>\n";
		$test->details .= "<td>$file</td><td>\n";

		$filePath = GetDataSourcePath() . $file;
		if (file_exists($filePath)) {
			$counterUrl = 0;
			$counterMID = 0;
			$fileLines = GL_file($filePath);
			for ($i = 0; $i < count($fileLines); ++$i) {
				$line = $fileLines[$i];
				$lineContent = preg_split("/\s/", $line);

				if (count($lineContent) >= 2) {
					$param = $lineContent[0];
					$value = $lineContent[1];
					switch ($param) {
						case "url":
							$counterUrl++;
							$test->details .= "url: <a target=\"_blank\" href=\"$value\">$value</a><br>\n";
							if ($value == "") {
								$test->details .= $errStr . "\"url\" value is not set.<br>\n";
								$error++;
							} else {
								$test->details .= "<span class=\"Verify\">Verify the URL</span><br>\n";
							}
							break;
						case "merchant-id":
							$counterMID++;
							$test->details .= "merchant-id: $value<br>\n";
							if ($value == "") {
								$test->details .= $errStr . "\"merchant-id\" value is not set.<br>\n";
								$error++;
							}
							break;
						default:
							break;
					}
				}
			}
			if ($counterUrl == 0) {
				$test->details .= $errStr . "\"url\" setting is not in the file.<br>\n";
				$error++;
			}
			if ($counterUrl > 1) {
				$test->details .= $errStr . "There are multiple \"url\" settings in the file.<br>\n";
				$error++;
			}
			if ($counterMID == 0) {
				$test->details .= $errStr . "\"merchant-id\" setting is not in the file.<br>\n";
				$error++;
			}
			if ($counterMID > 1) {
				$test->details .= $errStr . "There are multiple \"merchant-id\" settings in the file.<br>\n";
				$error++;
			}
		} else {
			$test->details .= "ERROR: file ($filePath) was not found.<br>\n";
		}

		$test->details .= "</td></tr>\n";
	}
	$test->details .= "</table>\n";
	closedir($handle);

	if ($error == 0) {
		$test->Pass("CustomMerchant settings seem to be fine.");
	} else {
		$test->Verify("CustomMerchant settings may have problem(s).");
	}

	if ($error == 0) {
		if ($counterEcp == 0) {
			$test->Verify("No CustomMerchant datasource (*.ecp) files found in datasources folder.");
		} else {
			$test->Verify("Verify url setting on the each hyperlink.");
		}
	} else {
		$test->Fail("Problem(s) were found with one or more CustomMerchant datasource file(s).");
	}

	return $test;
}

////$depth = array();
//
//function startElementXds($parser, $name, $attrs) {
//    global $depth;
//    for ($i = 0; $i < $depth[$parser]; $i++) {
//        print "  ";
//    }
//    print "$name\n";
////    $depth[$parser]++;
//}
//
//function endElementXds($parser, $name) {
//    global $depth;
//    $depth[$parser]--;
//}

function checkServerXdsFiles()
{
	$test = new Test("Check XML datasource files");

	$test->description = "This test checks the XML datasource file ";
	$test->description .= "in config/datasources folder.\n";

	$test->details = "";
	$error = 0;
	$counterXds = 0;
	$errStr = "<span class=\"Failed\">ERROR</span><br>\n";

	$folder = GetDataSourcePath();

	$test->details .= "<table border='1'>\n";
	$test->details .= "<tr><th>Data Source</th><th>Setting</th></tr>";

	$handle = opendir(GetDataSourcePath());
	while ($file = readdir($handle)) {
		if (ereg('(.*)\.xds$', $file, $regs)){
			$dataSource = $regs[1];
			$counterXds++;
		} else {
			continue;
		}
		
		$fullpath = $folder . "/" . $file;
		if (!($fp = fopen($fullpath, "r"))) {
			$error++;
			$test->details .= "<span class='failed'>Could not open XML input.</span><br>\n";
			continue;
		}

		$test->details .= "<tr>\n";
		$test->details .= "<td>$file</td><td>\n";

		$xml_parser = xml_parser_create();
		xml_set_element_handler($xml_parser, "startElement", "endElement");
		$thisFileError = 0;
		while ($data = fread($fp, 4096)) {
			if (!xml_parse($xml_parser, $data, feof($fp))) {
				$test->details .= sprintf( "<span class='failed'>XML error: %s at line %d.</span><br>\n",
					xml_error_string(xml_get_error_code($xml_parser)),
					xml_get_current_line_number($xml_parser));
				$thisFileError++;
			}
		}
		if ($thisFileError == 0) {
			// ToDO: more information
			$test->details .= "XML format is fine.";
		} else {
			$error++;
		}
		xml_parser_free($xml_parser);
		fclose($fp);
		$test->details .= "</td></tr>\n";
	}
	$test->details .= "</table>\n";

	if ($error == 0) {
		if ($counterXds == 0) {
			$test->Verify("No XML datasource (*.xds) files found in datasources folder.");
		} else {
			$test->Pass("XML datasource seems to be fine.");
		}
	} else {
		$test->Fail("Problem(s) were found with one or more XML datasource file(s).");
	}

	return $test;
}


function checkOther()
{
	$test = new Test("Check other issues");

	$test->description  = "If you are still having problems, here's a list of known issues you can check manually. ";

	$title = "";
	$symptom = "";
	$prescription = "";
	$test->details = "";

	$title = "POST versus GET";
	$symptom = "You get unexpected results or errors when you update records or link to detail pages.";
	$prescription = "Use the POST method, instead of the GET method, on forms with Dynamic Content actions.\n";
	$prescription .= "The GET method will not work.\n";
	$test->details .= checkOtherSubAddToTestDetails($title, $symptom, $prescription);

	$title = "HTML Tags in Database Records";
	$symptom = "&lt;TEXTAREA&gt; tag breaks layout";
	$prescription = "If records in your database include strings that can be interpreted as\n";
	$prescription .= "HTML tags (for example, <code>&lt;/textarea&gt;</code>, HTML rendering may fail.\n";
	$prescription .= "To avoid this problem, set the field's filter to \"Encode String as HTML\"\n";
	$prescription .= "in the Dynamic Bindings palette.\n";
	$test->details .= checkOtherSubAddToTestDetails($title, $symptom, $prescription);

	$title = "config/info folder";
	$symptom = "Security Issue";
	$prescription = "The files in the \"info\" folder (<code>config/info/*.*</code>)\n";
	$prescription .= "are used by GoLive to create PHP documents,\n";
	$prescription .= "but are not needed when you deploy your finished pages.\n";
	$prescription .= "You should not install the config/info files on your\n";
	$prescription .= "production server because of the authoring access they provide for the site.\n";
	$prescription .= "<p>\n";
	$prescription .= "You may wish to install these files temporarily to verify\n";
	$prescription .= "your production server's environment using this\n";
	$prescription .= "troubleshooting.php document,\n";
	$prescription .= "but should remove them after completing the tests.\n";
	$test->details .= checkOtherSubAddToTestDetails($title, $symptom, $prescription);

	$test->Verify("Check these issues manually.");

	return $test;
}


$TestResults = Array();

clearstatcache();	//	Clear file system cache

// perform each test and store resulting Test objects in TestResults
if (precheck() == 0) {
	$TestResults[] = checkPHPInfo();
	$TestResults[] = checkPHPIni();
	$TestResults[] = checkPHPFunctions();
	$TestResults[] = checkMultiByteSupport();
	$TestResults[] = checkServerConfigFiles();
	$TestResults[] = checkServerConfigWritePrivilege();
	$TestResults[] = checkSessionFileWritePrivilege();
	$TestResults[] = checkServerXMLFileFormat();
	$TestResults[] = checkServerConfigFilesReturnCode();
	$TestResults[] = checkServerConfigFriendsFile();
	$TestResults[] = checkServerMySQL();
	$TestResults[] = checkServerEcpFiles();
	$TestResults[] = checkServerXdsFiles();
	$TestResults[] = checkOther();

}

// write out the test summary, followed by the individual test details
// print(reportSummary($TestResults));
reportSummary();
reportDetail();

?>
	</body>
</html>
