<?php

/*
 * ADOBE SYSTEMS INCORPORATED
 * Copyright 2002 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE:  Notwithstanding the terms of the Adobe GoLive End User 
 * License Agreement, Adobe permits you to reproduce and distribute this 
 * file only as an integrated part of a web site created with Adobe 
 * GoLive software and only for the purpose of enabling your client to 
 * display their web site. All other terms of the Adobe license 
 * agreement remain in effect.
 */

RejectUnauthorizedCallers();
require_once("../include/mysql.runtime6.php");
$title = "Adobe GoLive (PHP) Troubleshooting Report";

// Test results are one of the following
$Test_Pass	 = 0;
$Test_Fail	 = 1;
$Test_Verify = 2;

// HTML strings for each test result
$Test_ResultStr = array(
	"<span class=\"passed\">Passed</span>",
	"<span class=\"failed\">Failed</span>",
	"<span class=\"verify\">Verify</span>");

// Directory scan error count
$directoryScanError = 0;

/*
 * True iff the troubleshooting page is in the config/info folder.
 */
function precheck()
{
	$error = 0;
	global $PHP_SELF;
	if (!ereg('config[\/]info[\/]troubleshooting', $PHP_SELF)) {
		$msg = "<span class='failed'>\n";
		$msg .= "This troubleshooting.php file needs to be under the config/info folder to work properly.\n";
		$msg .= "Please relocate it and try again. If troubleshooting.php is in config/info folder,\n";
		$msg .= "register_globals may be \"off\" in php.ini file. It needs to be \"on\" as default setting.\n";
		$msg .= "</span>\n";
		echo $msg;
		$error = -1;
	}
	return $error;
}

// --- Test Class -------------------------------------------------------------
// A simple object with instance variables containing the results.
// ----------------------------------------------------------------------------
class Test
{
	var $title;
	var $description;
	var $details;
	var $result;
	var $summary;

	function Test($title){
		global	$Test_Pass;
		$this->title		= $title;
		$this->description	= "";
		$this->details		= "";
		$this->result		= $Test_Pass;
		$this->summary		= "";
	}

	function Pass($summary){
		global	$Test_Pass;
		$this->result  = $Test_Pass;
		$this->summary = $summary;
	}

	function Fail($summary){
		global	$Test_Fail;
		$this->result  = $Test_Fail;
		$this->summary = $summary;
	}

	function Verify($summary){
		global	$Test_Verify;
		$this->result  = $Test_Verify;
		$this->summary = $summary;
	}

	function ResultStr(){
		global $Test_ResultStr;
		return $Test_ResultStr[$this->result];
	}
}


function reportSummary() {
	global $TestResults, $Test_ResultStr;

	$n = count($TestResults);
	$ntests = $n==1 ? '1 test' : "$n tests";
	echo "<hr>\r\n",
		"<h2>Test Summary</h2>\r\n",
		"<p><b>", $ntests, " performed.</b>\r\n",
		"<p><table border='1' cellpadding='4'>\r\n",
		"<tr><th>Test</th><th>Description</th><th>Results</th>",
		"<th>Summary</th></tr>\r\n";

	for ($i = 0; $i < count($TestResults); ++$i) {
		$test = &$TestResults[$i];
		$i1 = 1+$i;
		echo "<tr><td align='center'>",
			"<a href='#{$i1}'>", $i1, "</a>",
			"</td>\r\n",
			"<td>",
			"<a href='#{$i1}'>", $test->title, "</a>",
			"</td>\r\n",
			"<td>", $test->ResultStr(), "</td>\r\n",
			"<td>", $test->summary, "</td>\r\n",
			"\r\n";
	}

	echo "</table>\r\n",
		"<h3>How to read the test results</h3>\r\n",
		$Test_ResultStr[0], ":
The test was successful.
No further action is required.<br>\r\n",
		$Test_ResultStr[1], ":
Problem(s) were found.
Refer to the test report details for more information.<br>\r\n",
		$Test_ResultStr[2], ":
This may or may not be a problem, depending on your environment.
Read the report details for instructions on manually validating
these items.<br>\r\n";
}

function reportDetail() {
	global $TestResults;
	for ($i = 0; $i < count($TestResults); ++$i) {
		$test = &$TestResults[$i];
		echo "<hr><a name='", 1+$i, "'></a>\r\n",
			"<h2>Test ", 1+$i, ': ', $test->title, "</h2>\r\n",
			"<p>", $test->ResultStr(), ': ', $test->summary,
			"</p>\r\n",
			"<p><b>Description: </b>", $test->description,
			"</p>\r\n",
			$test->details,
			"<p><a href='#top'>Back to Top</a></p>\r\n";
	}
}

function onoff($t, $def=null) {
	if ($def===null)
		$def = $t;
	$r = $t ? 'on' : 'off';
	if ($t == $def)
		return $r;
	return "<span class='failed'>$r</span>";
}

/*
 * Sort routine for filenames.  Could be better localized.
 */
function fnamecmp($a, $b) {
	return strcmp($a, $b);
}


function checkOtherSubAddToTestDetails($title, $symptom, $prescription)
{
	$msg = "<div class='manual'>\n";
	$msg .= "<span class='manualTitle'>$title</span>\n";
	$msg .= "<p>\n";
	$msg .= "<span class='manualSymptom'><b>Symptom: </b>$symptom</span>\n";
	$msg .= "<p>\n";
	$msg .= "<span class='manualPrescription'><b>Prescription: </b>$prescription</span>\n";
	$msg .= "</div>\n";
	return $msg;
}

?>
